from django.test import TestCase
from tour.utils.import_string import import_string


class UtilTest(TestCase):
    """
    Tests the functions in the utils file
    """
    def test_import_string(self):
        """
        Tests all outcomes out import_string
        """
        # Make sure an invalid module path returns None
        self.assertIsNone(import_string('nope.nope'))

        # Make sure an invalid module name returns None
        self.assertIsNone(import_string('tour.nope'))

        # For test coverage, import a null value
        self.assertIsNone(import_string('tour.tests.mocks.mock_null_value'))



from django.http import HttpResponseRedirect
from tour.models import Tour


class TourStepMixin(object):
    """
    Provides a method for requiring navigation through the tour steps in order
    """
    def tour_should_redirect(self, tour_class, current_index, next_index):
        # check if tour is incomplete
        if tour_class.current_step_class:
            # check if the current step is later in the tour than the expected step
            if current_index >= 0 and next_index >= 0:
                if current_index > next_index:
                    return True
        else:
            # tour is complete, make sure we don't go backwards
            # check that the current page is a step of the tour and check if the current page
            # is the next page of the tour (which should be the last page)
            if current_index >= 0 and current_index != next_index:
                return True
        return False

    def get_user_tour_class(self, request):
        # Get the tour class for the user
        tour_class = Tour.objects.get_for_user(request.user)
        if tour_class is None:
            tour_class = Tour.objects.get_recent_tour(request.user)
        return tour_class

    def get_tour_redirect_url(self, request):
        tour_class = self.get_user_tour_class(request)
        if not tour_class:
            return None

        # Determine the current step and expected step indices
        next_url = tour_class.get_next_url()
        url_list = tour_class.get_url_list()

        current_index = -1
        next_index = -1
        if request.path in url_list:
            current_index = url_list.index(request.path)
        if next_url in url_list:
            next_index = url_list.index(next_url)
        should_redirct = self.tour_should_redirect(tour_class, current_index, next_index)
        if should_redirct:
            return next_url
        return None

    def dispatch(self, request, *args, **kwargs):
        redirect_url = self.get_tour_redirect_url(request)
        if redirect_url:
            return HttpResponseRedirect(redirect_url)

        return super(TourStepMixin, self).dispatch(request, *args, **kwargs)



from django.contrib.auth.models import User
from django.db import models

from tour.utils.import_string import import_string


class TourManager(models.Manager):
    """
    Provides extra functionality for the Tour model
    """
    def get_for_user(self, user):
        """
        Checks if a tour exists for a user and returns the instantiated tour object
        """
        if not user.pk:
            return None
        queryset = self
        tours = queryset.filter(tourstatus__user=user, tourstatus__complete=False).order_by('-tourstatus__create_time')
        for tour in tours:
            tour_class = tour.load_tour_class()
            if tour_class.is_complete(user=user) is False:
                return tour_class
        return None

    def get_recent_tour(self, user):
        if not user.pk:
            return None
        tour = self.filter(tourstatus__user=user).order_by(
            'tourstatus__complete', '-tourstatus__complete_time').first()
        if tour:
            return tour.load_tour_class()
        return None

    def get_next_url(self, user):
        """
        Convenience method to get the next url for the specified user
        """
        tour_class = self.get_for_user(user)
        if not tour_class:
            tour_class = self.get_recent_tour(user)
        if tour_class:
            return tour_class.get_next_url()
        return None


class Tour(models.Model):
    """
    Container object for tour steps. Provides functionality for loading the tour logic class
    and fetching the steps in the correct order.
    """
    name = models.CharField(max_length=128, unique=True)
    tour_class = models.CharField(max_length=128, unique=True)
    users = models.ManyToManyField(User, through='TourStatus')
    complete_url = models.CharField(max_length=128, blank=True, null=True, default=None)

    objects = TourManager()

    # TODO: look into callable field app
    def load_tour_class(self):
        """
        Imports and returns the tour class.
        """
        return import_string(self.tour_class)(self)

    def get_steps(self, parent_step=None):
        """
        Returns the steps in order based on if there is a parent or not
        TODO: optimize this
        """
        all_steps = []
        steps = self.step_set.all().filter(parent_step=parent_step).order_by('id')
        for step in steps:
            all_steps.append(step)
            all_steps += self.get_steps(step)
        return all_steps


class Step(models.Model):
    """
    Represents one step of the tour that must be completed. The custom logic is implemented
    in the class specified in step_class
    """
    name = models.CharField(max_length=128, unique=True)
    url = models.CharField(max_length=128, null=True, blank=True)
    tour = models.ForeignKey(Tour)
    parent_step = models.ForeignKey('self', null=True, related_name='child_steps')
    step_class = models.CharField(max_length=128, unique=True)

    def load_step_class(self):
        """
        Imports and returns the step class.
        """
        return import_string(self.step_class)(self)


class TourStatus(models.Model):
    """
    This is the model that represents the relationship between a user and a tour. Keeps
    track of whether the tour has been completed by a user.
    """
    tour = models.ForeignKey(Tour)
    user = models.ForeignKey(User)
    complete = models.BooleanField(default=False)
    create_time = models.DateTimeField(auto_now_add=True)
    complete_time = models.DateTimeField(null=True, blank=True, default=None)



from django.contrib.auth.models import User
from django.template import Template, Context
from tour.tests.mocks import MockTour, MockRequest, MockStep1, MockStep2
from tour.tests.tour_tests import BaseTourTest


class TemplateTagTest(BaseTourTest):
    """
    Tests the functionality of the template tag
    """
    def test_tour_tag(self):
        """
        Verifies that a tour gets displayed when a user has a tour
        """
        # Verifies that the tour template does not get rendered without a user
        MockTour.create()
        test_template = Template('{% load tour_tags %}{% tour_navigation %}')
        context = Context({
            'request': MockRequest(User(), '/mock/path'),
        })
        self.assertEqual('', test_template.render(context))

        # Verifies that the tour template does not get rendered if a user doesn't have a tour
        self.login_user1()
        context = Context({
            'request': MockRequest(self.test_user, '/mock/path'),
        })
        self.assertEqual('', test_template.render(context).strip())

        # Verifies that the tour template gets rendered if a user has a tour
        MockTour.add_user(self.test_user)
        test_template = Template('{% load tour_tags %}{% tour_navigation %}')
        context = Context({
            'request': MockRequest(self.test_user, '/mock/path'),
        })
        self.assertTrue('tour-wrap' in test_template.render(context))

        # Verify that the current class gets applied
        context = Context({
            'request': MockRequest(self.test_user, 'mock1'),
        })
        self.assertTrue('current' in test_template.render(context))

        # Verify that the complete class gets applied
        MockStep1.complete = True
        context = Context({
            'request': MockRequest(self.test_user, 'mock2'),
        })
        self.assertTrue('complete' in test_template.render(context))

        # Make sure no tour gets rendered when it is complete
        MockStep2.complete = True
        test_template = Template('{% load tour_tags %}{% tour_navigation %}')
        context = Context({
            'request': MockRequest(self.test_user, '/mock/path'),
        })
        self.assertTrue('tour-wrap' not in test_template.render(context))

        # Makes sure that the tour does get displayed if the always_show flag is on
        test_template = Template('{% load tour_tags %}{% tour_navigation always_show=True %}')
        context = Context({
            'request': MockRequest(self.test_user, '/mock/path'),
        })
        self.assertTrue('tour-wrap' in test_template.render(context))

        # Verify no errors for missing request object
        context = Context({})
        self.assertEqual('', test_template.render(context))

    def test_step_classes(self):
        self.login_user1()
        MockTour.add_user(self.test_user)

        # Test that the second step has an available class but not a complete class
        MockStep1.complete = True
        MockStep2.complete = True
        test_template = Template('{% load tour_tags %}{% tour_navigation always_show=True %}')
        context = Context({
            'request': MockRequest(self.test_user, 'mock1'),
        })
        rendered_content = self.render_and_clean(test_template, context)
        expected_str = '<a href="mock2" class="step-circle available ">'
        self.assertTrue(expected_str in rendered_content)

    def test_tour_title(self):
        """
        Makes sure the appropriate title gets displayed for the tour title
        """
        self.login_user1()
        MockTour.add_user(self.test_user)

        test_template = Template('{% load tour_tags %}{% tour_navigation %}')
        context = Context({
            'request': MockRequest(self.test_user, 'mock1'),
        })
        rendered_content = self.render_and_clean(test_template, context)

        # Make sure the current step is displayed
        expected_html = '<div class="tour-name">{0}</div>'.format(MockStep1.name)
        self.assertTrue(expected_html in rendered_content)

        # Make sure the tour title is displayed
        context = Context({
            'request': MockRequest(self.test_user, 'mock0'),
        })
        rendered_content = self.render_and_clean(test_template, context)
        expected_html = '<div class="tour-name">{0}</div>'.format(MockTour.name)
        self.assertTrue(expected_html in rendered_content)

    def render_and_clean(self, template, context):
        # render the template
        rendered_content = template.render(context).strip()
        # remove tabs
        rendered_content = rendered_content.replace('    ', '')
        # remove new lines
        rendered_content = rendered_content.replace('\n', '')
        return rendered_content



from django.conf.urls import patterns, include, url
from tastypie.api import NamespacedApi

from .api import TourResource, StepResource

api = NamespacedApi(api_name='api', urlconf_namespace='tour')
api.register(TourResource())
api.register(StepResource())

urlpatterns = patterns(
    '',
    url(r'^', include(api.urls, namespace='tour')),
)



from django.http import HttpResponse
from django.views.generic import View
from tour.tours import BaseStep, BaseTour
from tour.views import TourStepMixin


mock_null_value = None


class MockView(TourStepMixin, View):
    def get(self, request):
        return HttpResponse('ok')


class MockRequest(object):
    def __init__(self, user=None, path=None, params=None, method=None):
        self.user = user
        self.path = path
        self.method = method or 'get'
        self.GET = params or {}


class MockStep(BaseStep):
    complete = False

    def is_complete(self, user=None):
        return self.complete


class MockStep1(MockStep):
    step_class = 'tour.tests.mocks.MockStep1'
    name = 'Mock Step 1'
    complete = False

    @classmethod
    def get_url(cls):
        return 'mock1'


class MockStep2(MockStep):
    step_class = 'tour.tests.mocks.MockStep2'
    name = 'Mock Step 2'
    complete = False

    @classmethod
    def get_url(cls):
        return 'mock2'


class MockStep3(MockStep):
    step_class = 'tour.tests.mocks.MockStep3'
    name = 'Mock Step 3'
    complete = False
    parent_step = MockStep1

    @classmethod
    def get_url(cls):
        return 'mock3'


class MockStep4(MockStep):
    step_class = 'tour.tests.mocks.MockStep4'
    name = 'Mock Step 4'
    complete = False
    parent_step = MockStep1

    @classmethod
    def get_url(cls):
        return 'mock4'


class EmptyStep(MockStep):
    pass


class NoUrlStep(MockStep):
    step_class = 'tour.tests.mocks.NoUrlStep'
    name = 'No Url Step'
    complete = False


class MockTour(BaseTour):
    tour_class = 'tour.tests.mocks.MockTour'
    name = 'Mock Tour'
    complete_url = 'mock_complete1'
    steps = [
        MockStep1,
        MockStep2,
    ]


class MockTour2(BaseTour):
    tour_class = 'tour.tests.mocks.MockTour2'
    name = 'Mock Tour 2'
    complete_url = 'mock_complete2'
    parent_tour = MockTour
    steps = [
        MockStep3,
        MockStep4,
    ]


class MockTour3(BaseTour):
    tour_class = 'tour.tests.mocks.MockTour3'
    name = 'Mock Tour 3'
    complete_url = 'mock_complete3'
    steps = [
        EmptyStep
    ]


class MockTour4(BaseTour):
    name = 'Mock Tour 4'
    complete_url = 'mock_complete4'
    steps = [
        EmptyStep
    ]


class CompleteTour(BaseTour):
    tour_class = 'tour.tests.mocks.CompleteTour'
    name = 'Complete Tour'
    complete_url = 'mock_complete'
    steps = [
        MockStep1
    ]

    def is_complete(self, user=None):
        return True



from tour.tests.mocks import MockTour, MockRequest, MockView, MockStep1, MockStep2
from tour.tests.tour_tests import BaseTourTest


class ViewTest(BaseTourTest):
    """
    Tests the functionality of all tour views and mixins
    """
    def test_tour_step_mixin(self):
        """
        Verifies that a user can't go to steps out of order and can't go to other steps
        after the tour is complete
        """
        # do request when there should be no tour
        mock_request = MockRequest(self.test_user, 'mock2', {})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(200, response.status_code)

        MockTour.add_user(self.test_user)

        # request page that isn't in the tour before tour is complete
        mock_request = MockRequest(self.test_user, 'mock-fake', {})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(200, response.status_code)

        # do request to second step when we should be on first
        mock_request = MockRequest(self.test_user, 'mock2', {})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(302, response.status_code)
        self.assertEqual('mock1', response.url)

        # do request to first step
        mock_request = MockRequest(self.test_user, 'mock1', {})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(200, response.status_code)

        # complete tour and try to go to first step
        MockStep1.complete = True
        MockStep2.complete = True
        mock_request = MockRequest(self.test_user, 'mock1', {})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(302, response.status_code)
        self.assertEqual('mock_complete1', response.url)

        # request page that isn't in the tour when tour is complete
        mock_request = MockRequest(self.test_user, 'mock-fake', {})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(200, response.status_code)



def import_string(module_string):
    parts = module_string.split('.')
    assert len(parts) > 1

    path = '.'.join(parts[:-1])
    module_name = parts[-1]
    file_name = parts[-2]

    try:
        module_path = __import__(path, globals(), locals(), [file_name])
    except ImportError:
        return None

    if not module_path or not hasattr(module_path, module_name):
        return None

    module = getattr(module_path, module_name)
    if not module:
        return None

    return module



from django.contrib.auth.models import User
from django.test import TestCase
from mock import patch
from tour.exceptions import MissingStepClass, MissingTourClass

from tour.models import Tour, Step, TourStatus
from tour.tests.mocks import (
    MockTour, MockStep1, MockStep2, MockTour2, MockStep3, MockStep4, MockTour3, MockTour4, CompleteTour)
from tour.tours import BaseStep


class BaseTourTest(TestCase):
    """
    Provides basic setup for tour tests like creating users
    """
    def setUp(self):
        super(BaseTourTest, self).setUp()
        self.reset_mock_tour_states()
        self.test_user = User.objects.create_user('test', 'test@gmail.com', 'test')
        self.test_user2 = User.objects.create_user('test2', 'test2@gmail.com', 'test2')

    def login_user1(self):
        self.client.login(username='test', password='test')

    def login_user2(self):
        self.client.login(username='test2', password='test2')

    def reset_mock_tour_states(self):
        mock_steps = [MockStep1, MockStep2, MockStep3, MockStep4]
        for mock_step in mock_steps:
            mock_step.complete = False


class TourTest(BaseTourTest):
    """
    Tests the functionality of the Tour and Step classes
    """
    def test_empty_url(self):
        """
        Verifies that the base step does not return a url
        """
        self.assertIsNone(BaseStep.get_url())

    def test_is_complete(self):
        """
        Verifies that the base step is never complete
        """
        step = Step(name=BaseStep.name, url=BaseStep.get_url())
        self.assertFalse(BaseStep(step).is_complete(self.test_user))

    def test_exceptions(self):
        """
        Verifies that an exception is raised when there is no step class or tour class
        """
        with self.assertRaises(MissingStepClass):
            MockTour3.create()

        with self.assertRaises(MissingTourClass):
            MockTour4.create()

    def test_create(self):
        """
        Verifies that the tour gets created only once
        """
        # Create a single tour
        MockTour.create()
        self.assertEqual(1, Tour.objects.all().count())
        self.assertEqual(len(MockTour.steps), Step.objects.all().count())

        # Try to create a duplicate
        MockTour.create()
        self.assertEqual(1, Tour.objects.all().count())
        self.assertEqual(len(MockTour.steps), Step.objects.all().count())

        # Create a tour with a parent and make sure only the steps get added to the parent tour
        MockTour2.create()
        self.assertEqual(1, Tour.objects.count())
        self.assertEqual(4, Step.objects.count())

    def test_delete(self):
        """
        Verifies that a tour and its steps get deleted
        """
        MockTour.create()
        MockTour2.create()
        self.assertEqual(1, Tour.objects.count())
        self.assertEqual(4, Step.objects.count())

        MockTour2.delete()
        self.assertEqual(1, Tour.objects.count())
        self.assertEqual(2, Step.objects.count())

        MockTour.delete()
        self.assertEqual(0, Tour.objects.count())
        self.assertEqual(0, Step.objects.count())

    def test_mark_complete(self):
        """
        Verifies that tours get marked as complete per user or for all users
        """
        MockTour.add_user(self.test_user)
        MockTour.add_user(self.test_user2)
        self.assertEqual(2, TourStatus.objects.filter(complete=False).count())
        tour_class = Tour.objects.get_for_user(self.test_user)
        tour_class.mark_complete(user=self.test_user)
        self.assertEqual(1, TourStatus.objects.filter(complete=False).count())
        # make sure marking complete again returns false
        self.assertFalse(tour_class.mark_complete(user=self.test_user))

        MockTour.add_user(self.test_user)
        self.assertEqual(2, TourStatus.objects.filter(complete=False).count())
        tour_class = Tour.objects.get().load_tour_class()
        tour_class.mark_complete()
        self.assertEqual(0, TourStatus.objects.filter(complete=False).count())

    def test_user_functions(self):
        """
        Makes sure a tour is created when adding a user to a tour that doesn't exist
        Verifies that a user can be assigned a tour only once
        Tests fetching an active tour for a user
        """
        self.assertIsNone(Tour.objects.get_for_user(User()))
        self.assertIsNone(Tour.objects.get_recent_tour(User()))
        self.assertIsNone(Tour.objects.get_for_user(self.test_user))

        # Create a single tour
        MockTour.create()

        # Add a user
        MockTour.add_user(self.test_user)
        self.assertEqual(1, TourStatus.objects.all().count())

        # Delete all tours and make sure a tour is created when adding a user
        Tour.objects.all().delete()
        self.assertEqual(0, TourStatus.objects.all().count())
        MockTour.add_user(self.test_user)
        self.assertEqual(1, TourStatus.objects.all().count())

        # Try to add the user again
        MockTour.add_user(self.test_user)
        self.assertEqual(1, TourStatus.objects.all().count())

        # Fetch the tour
        tour_class = Tour.objects.get_for_user(self.test_user)
        self.assertIsNotNone(tour_class)

        # Set the tours as complete
        MockStep1.complete = True
        MockStep2.complete = True

        # Check if MockTour is complete
        tour_class = Tour.objects.get_for_user(self.test_user)
        self.assertIsNone(tour_class)

        # Make sure we can get the most recent tour
        tour_class = Tour.objects.get_recent_tour(self.test_user)
        self.assertIsNotNone(tour_class)

        # Make sure a second tour will be added
        MockTour.add_user(self.test_user)
        self.assertEqual(2, TourStatus.objects.all().count())
        self.assertEqual(1, TourStatus.objects.filter(complete=False).count())

    def test_nested_tour(self):
        """
        Verifies that records get created properly for nested tours
        """
        MockTour.create()
        MockTour2.create()
        self.assertEqual(4, Step.objects.all().count())
        steps = list(Step.objects.all().order_by('id'))
        self.assertEqual(steps[0].parent_step, None)
        self.assertEqual(steps[1].parent_step, None)
        self.assertEqual(steps[2].parent_step, steps[0])
        self.assertEqual(steps[3].parent_step, steps[0])

    def test_url(self):
        """
        Verifies that the tour returns the correct next url
        """
        # Create the tours
        MockTour.create()
        MockTour2.create()

        # Add a user
        MockTour.add_user(self.test_user)

        # Fetch the tour
        tour_class = Tour.objects.get_for_user(self.test_user)

        # Get the url
        url = tour_class.get_next_url()
        self.assertEqual('mock1', url)

        MockStep1.complete = True
        tour_class = Tour.objects.get_for_user(self.test_user)
        url = tour_class.get_next_url()
        self.assertEqual('mock3', url)

        MockStep3.complete = True
        tour_class = Tour.objects.get_for_user(self.test_user)
        url = tour_class.get_next_url()
        self.assertEqual('mock4', url)

        MockStep4.complete = True
        tour_class = Tour.objects.get_for_user(self.test_user)
        url = tour_class.get_next_url()
        self.assertEqual('mock2', url)

        MockStep2.complete = True
        tour_class = Tour.objects.get_for_user(self.test_user)
        self.assertIsNone(tour_class)

        # Check that no url is returned for a complete tour
        Tour.objects.all().delete()
        Step.objects.all().delete()
        CompleteTour.add_user(self.test_user)
        tour_class = Tour.objects.get().load_tour_class()
        self.assertTrue(tour_class.is_complete())
        url = tour_class.get_next_url()
        self.assertEqual('mock_complete', url)

    def test_url_list(self):
        """
        Verifies that the tour builds the url list correctly
        """
        # Create the tours
        MockTour.create()
        MockTour2.create()

        # Add a user
        MockTour.add_user(self.test_user)

        # Fetch the tour
        tour_class = Tour.objects.get_for_user(self.test_user)

        urls = tour_class.get_url_list()
        expected_urls = ['mock1', 'mock3', 'mock4', 'mock2']
        self.assertEqual(expected_urls, urls)

    def test_next_url(self):
        """
        Verifies that the tour manager's next url method works
        """
        self.assertIsNone(Tour.objects.get_next_url(self.test_user))
        MockTour.add_user(self.test_user)
        self.assertEqual('mock1', Tour.objects.get_next_url(self.test_user))

    def test_updates(self):
        """
        Verifies that when tour or step data are updated, their records get updated as well.
        Also verifies that when steps of a tour change, the db reflects that
        """
        # create tour and add user
        MockTour.add_user(self.test_user)

        # change the name
        MockTour.name = 'Changed Tour Name'

        # fetch the tour like it is going to be displayed
        tour_class = Tour.objects.get_for_user(self.test_user)

        # the tour name should have been changed
        self.assertEqual(tour_class.tour.name, MockTour.name)

        # change a step name
        MockStep1.name = 'Changed Step Name'
        with patch('tour.tests.mocks.MockStep1.get_url') as mock_get_url:
            mock_get_url.return_value = 'different_url'

            # load the tour
            tour_class = Tour.objects.get_for_user(self.test_user)
            self.assertEqual(tour_class.current_step_class.step.name, MockStep1.name)
            self.assertEqual(tour_class.current_step_class.step.url, MockStep1.get_url())



"""
Custom template tags for displaying tour navigation
"""
from copy import deepcopy
import json

from django import template
from django.template.loader import get_template

from tour.api import TourResource
from tour.models import Tour


register = template.Library()


class TourNavNode(template.Node):
    def __init__(self, always_show=False):
        self.always_show = always_show

    def get_tour_class(self, request):
        # Check for any tours
        tour_class = Tour.objects.get_for_user(request.user)
        if not tour_class and self.always_show:
            tour_class = Tour.objects.get_recent_tour(request.user)
        if self.always_show:
            mutable_get = deepcopy(request.GET)
            mutable_get['always_show'] = True
            request.GET = mutable_get
        return tour_class

    def get_tour_dict(self, tour_class, request):
        if tour_class:
            # Serialize the tour and its steps
            tour = tour_class.tour
            tour_resource = TourResource()
            tour_bundle = tour_resource.build_bundle(obj=tour, request=request)
            tour_data = tour_resource.full_dehydrate(tour_bundle)
            tour_json = tour_resource.serialize(None, tour_data, 'application/json')
            tour_dict = json.loads(tour_json)

            # Set the step css classes
            previous_steps_complete = True
            is_after_current = False
            tour_dict['display_name'] = tour.name
            for step_dict in tour_dict['steps']:
                cls = ''
                if step_dict['url'] == request.path:
                    cls += 'current '
                    step_dict['current'] = True
                    tour_dict['display_name'] = step_dict['name']
                    is_after_current = True
                if not previous_steps_complete:
                    cls += 'incomplete unavailable '
                    step_dict['url'] = '#'
                elif not step_dict['complete']:
                    cls += 'incomplete available '
                    previous_steps_complete = False
                elif is_after_current:
                    cls += 'available '
                else:
                    cls += 'complete available '
                step_dict['cls'] = cls

            return tour_dict
        return {}

    def render(self, context):
        if 'request' in context and hasattr(context['request'], 'user'):
            # Make sure this isn't the anonymous user
            if not context['request'].user.id:
                return ''

            tour_class = self.get_tour_class(context['request'])
            context['tour'] = self.get_tour_dict(tour_class, context['request'])

            # Load the tour template and render it
            tour_template = get_template('tour/tour_navigation.html')
            return tour_template.render(context)
        return ''


@register.simple_tag(takes_context=True)
def tour_navigation(context, **kwargs):
    """
    Tag to render the tour nav node
    """
    return TourNavNode(always_show=kwargs.get('always_show', False)).render(context)



import datetime
from .models import Tour, Step, TourStatus
from tour.exceptions import MissingStepClass, MissingTourClass


class BaseStep(object):
    """
    Base step class that handles the creation of step records and determines when the step is complete
    """
    step_class = None
    name = None
    parent_step = None

    def __init__(self, step):
        self.step = step

        # Sync up data
        if self.name != self.step.name or self.__class__.get_url() != self.step.url:
            self.step.name = self.name
            self.step.url = self.__class__.get_url()
            self.step.save()

    @classmethod
    def get_url(cls):
        """
        This is meant to be overridden by subclasses to return the url of the step.
        """
        return None

    @classmethod
    def create(cls, tour):
        """
        Creates the step record under the specified tour
        """
        if cls.step_class is None:
            raise MissingStepClass('Step {0} needs a step_class in order to call create'.format(cls))

        # Get parent step if needed
        parent_step = None
        if cls.parent_step:
            parent_step = Step.objects.get(step_class=cls.parent_step.step_class)

        instance, created = Step.objects.get_or_create(
            step_class=cls.step_class, tour=tour, defaults=dict(
                name=cls.name,
                url=cls.get_url(),
                parent_step=parent_step,
            )
        )

        return instance

    def is_complete(self, user=None):
        """
        This is meant to be implemented in subclasses. This checks conditions to determine if the step is complete
        """
        return False


class BaseTour(object):
    """
    Base tour class that handles the creation of tour records and determines when tours are complete.
    """
    tour_class = None
    name = None
    steps = []
    parent_tour = None
    complete_url = None

    def __init__(self, tour):
        self.current_step_class = None
        self.tour = tour

        # Sync up data
        if self.name != self.tour.name or self.complete_url != self.tour.complete_url:
            self.tour.name = self.name
            self.tour.complete_url = self.complete_url
            self.tour.save()

    @classmethod
    def create(cls):
        """
        Creates the tour record
        """
        if cls.tour_class is None:
            raise MissingTourClass('Tour {0} needs a tour_class in order to call create'.format(cls))

        tour_class = cls.tour_class
        if cls.parent_tour:
            tour_class = cls.parent_tour.tour_class

        instance, created = Tour.objects.get_or_create(
            tour_class=tour_class, defaults=dict(
                name=cls.name,
                complete_url=cls.complete_url,
            )
        )

        # Make sure steps exist
        for step in cls.steps:
            step.create(instance)

        return instance

    @classmethod
    def delete(cls):
        """
        Deletes the tour and its steps. If the tour has a parent tour, then only the
        steps are deleted because there isn't a tour object.
        """
        # Build list of step classes
        step_classes = [step.step_class for step in cls.steps]
        Step.objects.filter(step_class__in=step_classes).delete()

        # Delete the tour if not a child tour
        if cls.parent_tour is None:
            Tour.objects.filter(tour_class=cls.tour_class).delete()

    @classmethod
    def add_user(cls, user):
        """
        Adds a relationship record for the user. Creates the tour record if it doesn't already exist
        """
        try:
            tour = Tour.objects.get(tour_class=cls.tour_class)
        except Exception:
            tour = cls.create()

        instance, created = TourStatus.objects.get_or_create(tour=tour, user=user, complete=False)
        return instance

    def get_next_url(self):
        """
        Gets the next url based on the current step. The tour should always be fetched with
        Tour.objects.get_for_user so the is_complete method is called and the current step class
        gets set.
        """
        if self.current_step_class:
            return self.current_step_class.step.url
        return self.complete_url

    def get_url_list(self):
        """
        Returns a flattened list of urls of all steps contained in the tour.
        """
        return [step.url for step in self.tour.get_steps() if step.url]

    def is_complete(self, user=None):
        """
        Checks the state of the steps to see if they are all complete
        """
        self.current_step_class = None

        # Check the state of all steps
        for step in self.tour.get_steps():
            step_class = step.load_step_class()
            if self.current_step_class is None and step_class.is_complete(user=user) is False:
                self.current_step_class = step_class
        if self.current_step_class:
            return False
        self.mark_complete(user=user)
        return True

    def mark_complete(self, user=None):
        """
        Marks the tour record as complete and saves it.
        """
        if user:
            queryset = self.tour.tourstatus_set.all().filter(tour=self.tour, user=user, complete=False)
        else:
            queryset = self.tour.tourstatus_set.all().filter(tour=self.tour, complete=False)
        # Check if there are any incomplete records
        if queryset.count():
            complete_time = datetime.datetime.utcnow()
            queryset.update(complete=True, complete_time=complete_time)
            return True
        return False



import json

from tastypie.authentication import SessionAuthentication
from tastypie.authorization import Authorization
from tastypie.resources import ModelResource

from .models import Tour, Step


class StepResource(ModelResource):
    """
    Api to access the Step model. Mostly used for serializing the steps in the tour dehydration
    """
    class Meta:
        queryset = Step.objects.all()
        filtering = {
            'id': ['exact'],
        }

        authorization = Authorization()
        authentication = SessionAuthentication()


class TourResource(ModelResource):
    """
    Api to access the Tour models.
    """
    class Meta:
        queryset = Tour.objects.all()
        filtering = {
            'id': ['exact'],
        }

        authorization = Authorization()
        authentication = SessionAuthentication()

    def dehydrate(self, bundle):
        """
        Provides serialization and flattening of the tour's steps
        """
        bundle = super(TourResource, self).dehydrate(bundle)
        if not bundle.request.GET.get('always_show'):
            is_complete = bundle.obj.load_tour_class().is_complete(user=bundle.request.user)
            if is_complete:
                return None
        steps = bundle.obj.get_steps()
        serialized = []
        step_resource = StepResource()
        for step in steps:
            step_bundle = step_resource.build_bundle(obj=step, request=bundle.request)
            step_data = step_resource.full_dehydrate(step_bundle)
            step_json = step_resource.serialize(None, step_data, 'application/json')
            step_dict = json.loads(step_json)
            step_class = step.load_step_class()
            step_dict['complete'] = step_class.is_complete(user=bundle.request.user)
            if step_dict['url']:
                serialized.append(step_dict)
        bundle.data['steps'] = serialized
        return bundle

    def alter_list_data_to_serialize(self, request, data):
        """
        Filter out any tours that are null. A tour will be returned as null if it is complete
        """
        data = super(TourResource, self).alter_list_data_to_serialize(request, data)
        data['objects'] = [obj for obj in data['objects'] if obj is not None]
        return data

    def apply_filters(self, request, applicable_filters):
        """
        Automatically filter the tour resource to return incomplete tours for a specific user
        """
        base_object_list = super(TourResource, self).apply_filters(request, applicable_filters)
        if request.GET.get('always_show'):
            base_object_list = base_object_list.filter(tourstatus__user=request.user)
        else:
            base_object_list = base_object_list.filter(tourstatus__user=request.user, tourstatus__complete=False)
        return base_object_list



import json

from django.core.urlresolvers import reverse

from tour.models import Tour
from tour.tests.mocks import MockTour, MockStep1, MockStep2, NoUrlStep
from tour.tests.tour_tests import BaseTourTest


class ApiTest(BaseTourTest):

    def test_fetch_tour(self):
        self.login_user1()
        url = reverse('tour:api_dispatch_list', kwargs={'resource_name': 'tour', 'api_name': 'api'})
        MockTour.create()

        # Make sure there are no tour objects returned
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        json_response = json.loads(response.content)
        objects = json_response.get('objects')
        self.assertEqual(0, len(objects))

        # Make sure there is one tour object returned
        MockTour.add_user(self.test_user)
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        json_response = json.loads(response.content)
        objects = json_response.get('objects')
        self.assertEqual(1, len(objects))

        # Create a tour for a second user and make sure only the one tour is returned
        MockTour.add_user(self.test_user2)
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        json_response = json.loads(response.content)
        objects = json_response.get('objects')
        self.assertEqual(1, len(objects))

        # Complete the tour for user 1
        MockStep1.complete = True
        MockStep2.complete = True
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        json_response = json.loads(response.content)
        objects = json_response.get('objects')
        self.assertEqual(0, len(objects))

        # make sure it can be returned with a param
        response = self.client.get(url, data={'always_show': 1})
        self.assertEqual(200, response.status_code)
        json_response = json.loads(response.content)
        objects = json_response.get('objects')
        self.assertEqual(1, len(objects))

        # Log in as user 2 and make sure there is still a tour
        self.login_user2()
        MockStep1.complete = False
        MockStep2.complete = False
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        json_response = json.loads(response.content)
        objects = json_response.get('objects')
        self.assertEqual(1, len(objects))

        # Make sure a step with no url is ignored
        tour = Tour.objects.get()
        NoUrlStep.create(tour)
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        json_response = json.loads(response.content)
        objects = json_response.get('objects')
        self.assertEqual(len(MockTour.steps), len(objects[0].get('steps')))




class MissingStepClass(Exception):
    """
    This exception is meant to be raised when a step is being created but it does not
    have a class defined
    """
    pass


class MissingTourClass(Exception):
    """
    This exception is meant to be raised when a tour is being created but it does not
    have a class defined
    """
    pass
